//
//  TweetsTableViewController.swift
//  Busca Pios
//
//  Created by Santiago Pavón on 15/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit

private let TWITTER_SEARCH_URL = "https://api.twitter.com/1.1/search/tweets.json"


class TweetsTableViewController: UITableViewController {

    // String a buscar en Twitter
    var query: String?
    
    // Los tweets encontrados tras la busqueda.
    // Es un array de objetos Tweet.
    var tweets = [Tweet]()
    
    // Almacen de profile images.
    // La clave es un String con el URL, y el valor es un objeto UIImage.
    var images = [String:UIImage]()
    
    // URL Session para descargar tweets
    var session: NSURLSession!
    
    // Estado del boton para bajar mas tweets.
    var downloadButtonEnabled: Bool = false {
        didSet {
            if let cell = self.tableView.cellForRowAtIndexPath(NSIndexPath(forRow: 0, inSection: 1)) as? NextPageCell {
                cell.downloadButton.enabled = downloadButtonEnabled
            }
        }
    }
    
    // Query para bajar la siguiente pagina de Tweets.
    var current_query: String?
    
    override func viewDidLoad() {
        super.viewDidLoad()
                
        //---
     
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 110
        
        //---
        
        autenticaYDescarga()
    }

    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        
        images = [:]  // Elimina las imagenes guardadas.
    }

    
    private func autenticaYDescarga() {
        self.title = "Autenticando ..."
        incrNetActivity()
        
        OAuthToken.getAccessToken() {(access_token: String?) in
            
            dispatch_async(dispatch_get_main_queue(), {
                
                self.decrNetActivity()
                
                if access_token != nil {
                    self.title = "Autenticado"
                    
                    self.createSession(access_token!)
                    self.getTweets()
                } else {
                    // No pudo realizarse la conexion o la descarga fallo.
                    self.title = "Error autenticando";
                }
            })
        }
    }
    
    // MARK: - URL Session para bajar los Tweets

    
    private func createSession(access_token: String) {
        
        println("Creando sesion para bajar tweets.")
        
        var config = NSURLSessionConfiguration.defaultSessionConfiguration()
        
        // Añadimos la cabecera de autenticacion necesaria para acceder al API de busquedas de Twitter.
        config.HTTPAdditionalHeaders = ["Authorization": "Bearer \(access_token)"]
     
        session = NSURLSession(configuration: config)
    }
    
    
    // MARK: - Search Tweets
    

    
    /**
    *  Obtiene todos los tweet que encajan con el string buscado (self.query).
    *
    *  Los tweets encontrados se dividen en paginas que hay que descargarse una a una.
    *
    *  Los tweets encontrados se almacenan en self.tweets.
    */
    private func getTweets() {
        
        // Construye la primera query que hay que añadir a la URL.
        // Con esta query solo se baja la primera pagina de tweets.
        current_query = "?q=\(self.query!)&count=100"
    
        downloadNextPage()
    }
    
    // Descarga los tweets indicado por current_query.
    func downloadNextPage() {
        
        self.title = "Downloading ..."
        incrNetActivity()
        downloadButtonEnabled = false
        
        // Construimos la URL añadiendo la query current_query.
        if let pageURL = "\(TWITTER_SEARCH_URL)\(current_query!)".stringByAddingPercentEscapesUsingEncoding(NSUTF8StringEncoding) {

            println("Page URL = \(pageURL)")
            
            if let url = NSURL(string:pageURL) {
                
                let task = session.dataTaskWithURL(url, completionHandler: { (data: NSData!, res: NSURLResponse!, error: NSError!) in
                    
                    let code = (res as! NSHTTPURLResponse).statusCode
                    print("code = \(code) ")
                    if error == nil && code == 200 {
                        // Procesamos el JSON recibido.
                        self.addJSONData(data)
                    } else {
                        dispatch_async(dispatch_get_main_queue(), {
                            self.decrNetActivity()
                            self.title = "Error 1"
                        })
                    }
                })
                task.resume()
                return
            }
        }
        self.decrNetActivity()
        self.title = "Error 2"
    }
    

    /**
    *  Dado un NSData con el JSON devuelto por el API de busquedas de Twitter, extrae el JSON, y
    *  crea objetos Tweet que añade al array self.tweets.
    *
    *  El JSON es un diccionario con dos claves: "statuses" y "search_metadata".
    *
    *  El valor asociado a la clave "statuses" es un array con los tweets encontrados.
    *
    *  Cada tweet es un diccionario. En esta aplicacion solo cogemos los valores asociados a las claves
    *  "text" y "user". El valor de "text" es el mensaje de tweet. El Valor de "user" es otro diccionario
    *  del que tomamos los valores asociados a las claves "name" y "profile_image_url", que son el nombre
    *  de autor y la url a su foto.
    *
    *  El valor asociado a la clave "search_metadata" es diccionaro del que solo nos interesa el valor
    *  asociado a la clave "next_results". Es la query a utilizar para bajar la siguiente pagina de tweets.
    *
    *  El parametro jsonData es un NSData conteniendo el JSON devuelto por twitter.
    */
    private func addJSONData(jsonData: NSData) {
        
        var err: NSError?
        if let dic = NSJSONSerialization.JSONObjectWithData(jsonData, options: .allZeros, error: &err) as? [String:AnyObject] {
            
            // println(dic)
            
            // Extrae los tweets y los almacena en self.tweets.
            // Y actualiza la Table View.
            for tweetDic in dic["statuses"] as! [[String:AnyObject]] {
                let text  = tweetDic["text"] as! String
                let username = (tweetDic["user"] as! [String:AnyObject])["name"] as! String
                let imageUrl = (tweetDic["user"] as! [String:AnyObject])["profile_image_url"] as! String
                
                let tweet = Tweet(text: text, username: username, imageUrl: imageUrl)
                
                dispatch_async(dispatch_get_main_queue(), {
                    self.tweets += [tweet]
                    let ip = NSIndexPath(forRow: self.tweets.count-1, inSection:0)
                    self.tableView.insertRowsAtIndexPaths([ip], withRowAnimation: .Automatic)
                })
            }
            
            dispatch_async(dispatch_get_main_queue(), {
                // La query a utilizar para bajar la siguiente pagina de tweets.
                self.current_query = nil
                if let nextQuery = (dic["search_metadata"] as! [String:AnyObject])["next_results"] as? String {
                    self.downloadButtonEnabled = true
                    self.current_query = nextQuery
                }
                
                self.decrNetActivity()
                self.title = self.query
            })
            
        } else {
            
            println("Error parsing JSON = \(err!.localizedDescription)")
            
            dispatch_async(dispatch_get_main_queue(), {
                self.decrNetActivity()
                self.title = "Error JSON"
            })
        }
    }
    
    // MARK: - Network Activity
    
    
    /**
    *  Contador usado para saber cuantas veces hemos ejecutado:
    *    UIApplication.sharedApplication().networkActivityIndicatorVisible = true
    *  Esta sentencia activa el indicador de actividad en la red.
    *  Mientras este contador sea mauor que cero no pararemos el indicador.
    */
    private var netActivityCredit = 0

    /**
    *  Activa el indicador de actividad de red.
    */
    private func incrNetActivity() {
        ++netActivityCredit
        UIApplication.sharedApplication().networkActivityIndicatorVisible = true
    }
    
    /**
    *  Desactiva el indicador de actividad de red.
    */
    private func decrNetActivity() {
        --netActivityCredit
        if netActivityCredit < 1 {
            UIApplication.sharedApplication().networkActivityIndicatorVisible = false
        }
    }
    
    
    // MARK: - Eventos: acccion del boton para bajar mas tweets
    
    @IBAction func nextPage() {
        
        downloadButtonEnabled = false

        downloadNextPage()
    }
    
 

    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 2
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        switch section {
        case 0: // Seccion con los tweets.
            return tweets.count
        default: // Secccion con el boton de bajar mas tweets.
            return 1
        }
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        switch indexPath.section {
        case 0:
            let cell = tableView.dequeueReusableCellWithIdentifier("Tweet Cell", forIndexPath: indexPath) as! TweetCell
            
            cell.msgLabel.text = tweets[indexPath.row].text
            cell.usernameLabel.text = tweets[indexPath.row].username
            
            let url = tweets[indexPath.row].imageUrl
            
            if let image = images[url] {
                cell.userImageView.image = image
            } else {
                cell.userImageView.image = nil
                
                if let url = NSURL(string: tweets[indexPath.row].imageUrl) {
                    
                    incrNetActivity()
                    
                    let task = session.dataTaskWithURL(url) { (data: NSData!, res: NSURLResponse!, error: NSError!) in
                        if error == nil && (res as! NSHTTPURLResponse).statusCode == 200 {
                            // La imagen descargada.
                            if let newImg = UIImage(data: data) {
                                dispatch_async(dispatch_get_main_queue(), {
                                    // Guardar la imagen
                                    self.images[self.tweets[indexPath.row].imageUrl] = newImg
                                    
                                    // Poner la nueva imagen en la fila de la tabla y actualizar la tabla
                                    if let cell = self.tableView.cellForRowAtIndexPath(indexPath) as? TweetCell {
                                        cell.userImageView.image = newImg;
                                    }
                                })
                            }
                        }
                        dispatch_async(dispatch_get_main_queue(), {
                            self.decrNetActivity()
                        })
                    }
                   task.resume()
                }
            }
            return cell
        default:
            let cell = tableView.dequeueReusableCellWithIdentifier("Next Page Cell", forIndexPath: indexPath) as! NextPageCell
            
            cell.downloadButton.enabled = downloadButtonEnabled

            return cell
        }
    }

}
